@@ -1,6 +1,6 @@
 /* markdown: a C implementation of John Gruber's Markdown markup language.
  *
- * Copyright (C) 2009 David L Parsons.
+ * Copyright (C) 2009 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -54,15 +54,13 @@ mkd_css(Document *d, char **res)
 	stylesheets(d->code, &f);
 			
 	if ( (size = S(f)) > 0 ) {
+	    /* null-terminate, then strdup() into a free()able memory
+	     * chunk
+	     */
 	    EXPAND(f) = 0;
-			/* HACK ALERT! HACK ALERT! HACK ALERT! */
-	    *res = T(f);/* we know that a T(Cstring) is a character pointer */
-			/* so we can simply pick it up and carry it away, */
-			/* leaving the husk of the Ctring on the stack */
-			/* END HACK ALERT */
+	    *res = strdup(T(f));
 	}
-	else
-	    DELETE(f);
+	DELETE(f);
 	return size;
     }
     return EOF;
