@@ -1,7 +1,7 @@
 /*
  * docheader -- get values from the document header
  *
- * Copyright (C) 2007 David L Parsons.
+ * Copyright (C) 2007 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -17,7 +17,12 @@
 static char *
 onlyifset(Line *l)
 {
-    char *ret = T(l->text) + l->dle;
+    char *ret;
+
+    if ( l->dle < 0 || l->dle >= S(l->text) )
+	return 0;
+
+    ret = T(l->text) + l->dle;
 
     return ret[0] ? ret : 0;
 }
