@@ -1,6 +1,6 @@
 /* markdown: a C implementation of John Gruber's Markdown markup language.
  *
- * Copyright (C) 2007 David L Parsons.
+ * Copyright (C) 2007 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -16,6 +16,7 @@
 #include "cstring.h"
 #include "markdown.h"
 #include "amalloc.h"
+#include "tags.h"
 
 typedef int (*stfu)(const void*,const void*);
 typedef void (*spanhandler)(MMIOT*,int);
@@ -132,13 +133,13 @@ Qchar(int c, MMIOT *f)
 {
     block *cur;
 
-    if ( S(f->Q) == 0 ) {
+    if ( S(f->Q) > 0 )
+	cur = &T(f->Q)[S(f->Q)-1];
+    else {
 	cur = &EXPAND(f->Q);
 	memset(cur, 0, sizeof *cur);
 	cur->b_type = bTEXT;
     }
-    else
-	cur = &T(f->Q)[S(f->Q)-1];
 
     EXPAND(cur->b_text) = c;
 
@@ -213,11 +214,12 @@ ___mkd_reparse(char *bfr, int size, mkd_flag_t* flags, MMIOT *f, char *esc)
     MMIOT sub;
     struct escaped e;
 
-    ___mkd_initmmiot(&sub, f->footnotes);
+    ___mkd_initmmiot(&sub, f->footnotes, flags);
 
-    COPY_FLAGS(sub.flags, f->flags);
-    if ( flags )
-	ADD_FLAGS(&sub.flags, flags);
+    ___mkd_or_flags(&sub.flags, &f->flags);
+
+    
+    
     sub.cb = f->cb;
     sub.ref_prefix = f->ref_prefix;
 
@@ -382,24 +384,47 @@ linkytitle(MMIOT *f, char quote, Footnote *ref)
 }
 
 
-/* extract a =HHHxWWW size from the input stream
+/* extract a =WxH size from the input stream
  */
 static int
 linkysize(MMIOT *f, Footnote *ref)
 {
-    int height=0, width=0;
+    Cstring height, width;
     int whence = mmiottell(f);
     int c;
 
+    CREATE(height);
+    CREATE(width);
+
     if ( isspace(peek(f,0)) ) {
 	pull(f);	/* eat '=' */
 
-	for ( c = pull(f); isdigit(c); c = pull(f))
-	    width = (width * 10) + (c - '0');
+	c = peek(f,1);
+
+	if ( isdigit(c) ) {
+	    for ( c = pull(f); isdigit(c); c = pull(f))
+		EXPAND(width) = c;
+	    if ( c == '%' ) {
+		EXPAND(width) = c;
+		c = pull(f);
+	    }
+	}
+	else
+	    pull(f);
 
 	if ( c == 'x' ) {
-	    for ( c = pull(f); isdigit(c); c = pull(f))
-		height = (height*10) + (c - '0');
+
+	    c = pull(f);
+	    if ( isdigit(c) ) {
+		while (isdigit(c) ) {
+		    EXPAND(height) = c;
+		    c = pull(f);
+		}
+		if ( c == '%' ) {
+		    EXPAND(height) = c;
+		    c = pull(f);
+		}
+	    }
 
 	    if ( isspace(c) )
 		c = eatspace(f);
@@ -410,6 +435,8 @@ linkysize(MMIOT *f, Footnote *ref)
 		return 1;
 	    }
 	}
+	DELETE(height);
+	DELETE(width);
     }
     mmiotseek(f, whence);
     return 0;
@@ -463,6 +490,22 @@ linkybroket(MMIOT *f, int image, Footnote *p)
 } /* linkybroket */
 
 
+/* extract a {}-delimited extended attribute from the input
+ * stream.
+ */
+static void
+linky_extended_attributes(MMIOT *f, struct footnote *p, int start)
+{
+    int c;
+
+    mmiotseek(f, start+1);
+
+    while ( (c = pull(f)) != '}' )
+	EXPAND(p->extended_attr) = c;
+    EXPAND(p->extended_attr) = 0;
+}
+
+
 /* extract a (-prefixed url from the input stream.
  * the label is either of the format `<link>`, where I
  * extract until I find a >, or it is of the format
@@ -502,9 +545,22 @@ linkyurl(MMIOT *f, int image, Footnote *p)
     if ( peek(f, 1) == ')' )
 	pull(f);
 
+    /* possible extended attributes? */
+    if ( is_flag_set(&f->flags, MKD_EXTENDED_ATTR) && (peek(f,1) == '{') ) {
+	int loc = mmiottell(f);
+	int size;
+
+	pull(f);
+
+	if ( (size = parenthetical('{','}',f)) != EOF )
+	    linky_extended_attributes(f, p, loc);
+	else
+	    mmiotseek(f, loc);
+    }
+
     ___mkd_tidy(&p->link);
 
-    if ( mayneedtotrim && (T(p->link)[S(p->link)-1] == '>') )
+    if ( mayneedtotrim && (S(p->link) > 0) && (T(p->link)[S(p->link)-1] == '>') )
 	--S(p->link);
 
     return 1;
@@ -559,7 +615,9 @@ typedef struct linkytype {
 } linkytype;
 
 static linkytype imaget = { 0, 0, "<img src=\"", "\"",
-			     1, " alt=\"", "\" />", { { [MKD_NOIMAGE] = 1, [MKD_TAGTEXT] = 1} }, IS_URL };
+			     1, " alt=\"", "\" />", { { [MKD_NOIMAGE] = 1,
+							[MKD_TAGTEXT] = 1,
+							[MKD_ALT_AS_TITLE] = 1 } }, IS_URL };
 static linkytype linkt  = { 0, 0, "<a href=\"", "\"",
 			     0, ">", "</a>", { {[MKD_NOLINKS] = 1} }, IS_URL };
 
@@ -609,10 +667,10 @@ printlinkyref(MMIOT *f, linkytype *tag, char *link, int size)
     Qstring(tag->link_pfx, f);
 
     if ( tag->kind & IS_URL ) {
-	if ( f->cb && f->cb->e_url && (edit = (*f->cb->e_url)(link, size, f->cb->e_data)) ) {
-
+	if ( f->cb && f->cb->e_url.func && (edit = (*f->cb->e_url.func)(link, size, f->cb->e_url.data)) ) {
 	    puturl(edit, strlen(edit), f, 0);
-	    if ( f->cb->e_free ) (*f->cb->e_free)(edit, f->cb->e_data);
+	    if ( f->cb->e_url.free )
+		(*f->cb->e_url.free)(edit, strlen(edit), f);
 	}
 	else
 	    puturl(link + tag->szpat, size - tag->szpat, f, 0);
@@ -626,10 +684,11 @@ printlinkyref(MMIOT *f, linkytype *tag, char *link, int size)
 
     Qstring(tag->link_sfx, f);
 
-    if ( f->cb && f->cb->e_flags && (edit = (*f->cb->e_flags)(link, size, f->cb->e_data)) ) {
+    if ( f->cb && f->cb->e_flags.func && (edit = (*f->cb->e_flags.func)(link, size, f->cb->e_flags.data)) ) {
 	Qchar(' ', f);
 	Qstring(edit, f);
-	if ( f->cb->e_free ) (*f->cb->e_free)(edit, f->cb->e_data);
+	if ( f->cb->e_flags.free )
+	    (*f->cb->e_flags.free) (edit, strlen(edit), f);
     }
 } /* printlinkyref */
 
@@ -652,7 +711,7 @@ extra_linky(MMIOT *f, Cstring text, Footnote *ref)
     if ( ref->fn_flags & REFERENCED )
 	return 0;
 
-    if ( is_flag_set(&f->flags,IS_LABEL) )
+    if ( is_flag_set(&f->flags, IS_LABEL) )
 	___mkd_reparse(T(text), S(text), &(linkt.flags), f, 0);
     else {
 	ref->fn_flags |= REFERENCED;
@@ -724,13 +783,18 @@ linkyformat(MMIOT *f, Cstring text, int image, Footnote *ref)
 	printlinkyref(f, tag, T(ref->link), S(ref->link));
 
 	if ( tag->WxH ) {
-	    if ( ref->height ) Qprintf(f," height=\"%d\"", ref->height);
-	    if ( ref->width ) Qprintf(f, " width=\"%d\"", ref->width);
+	    if ( S(ref->height) > 0 ) Qprintf(f," height=\"%s\"", T(ref->height));
+	    if ( S(ref->width) > 0 ) Qprintf(f, " width=\"%s\"", T(ref->width));
 	}
+	if ( S(ref->extended_attr) > 0 )
+	    Qprintf(f, " %s", T(ref->extended_attr));
 
-	if ( S(ref->title) ) {
+	if ( S(ref->title) || (is_flag_set(&f->flags, MKD_ALT_AS_TITLE) && is_flag_set(&tag->flags, MKD_ALT_AS_TITLE)) ) {
 	    Qstring(" title=\"", f);
-	    ___mkd_reparse(T(ref->title), S(ref->title), &tagtext, f, 0);
+	    if ( S(ref->title) )
+		___mkd_reparse(T(ref->title), S(ref->title), &tagtext, f, 0);
+	    else
+		___mkd_reparse(T(text), S(text), &tagtext, f, 0);
 	    Qchar('"', f);
 	}
 
@@ -770,26 +834,29 @@ linkylinky(int image, MMIOT *f)
 	else {
 	    int goodlink, implicit_mark = mmiottell(f);
 
-	    if ( isspace(peek(f,1)) )
-		pull(f);
-
-	    if ( peek(f,1) == '[' ) {
-		pull(f);	/* consume leading '[' */
-		goodlink = linkylabel(f, &key.tag);
+	    if ( is_flag_set(&f->flags, MKD_EXTRA_FOOTNOTE)
+		      && !is_flag_set(&f->flags, MKD_STRICT)
+		      && (!image)
+		      && S(name)
+		      && T(name)[0] == '^' ) {
+		extra_footnote = 1;
+		goodlink = 1;
 	    }
 	    else {
-		/* new markdown implicit name syntax doesn't
-		 * require a second []
-		 */
-		mmiotseek(f, implicit_mark);
-		goodlink = !is_flag_set(&f->flags, MKD_1_COMPAT);
+		if ( isspace(peek(f,1)) )
+		    pull(f);
 
-		if ( is_flag_set(&f->flags, MKD_EXTRA_FOOTNOTE)
-			  && !is_flag_set(&f->flags, MKD_STRICT)
-			  && (!image)
-			  && S(name)
-			  && T(name)[0] == '^' )
-		    extra_footnote = 1;
+		if ( peek(f,1) == '[' ) {
+		    pull(f);	/* consume leading '[' */
+		    goodlink = linkylabel(f, &key.tag);
+		}
+		else {
+		    /* new markdown implicit name syntax doesn't
+		     * require a second []
+		     */
+		    mmiotseek(f, implicit_mark);
+		    goodlink = !is_flag_set(&f->flags, MKD_1_COMPAT);
+		}
 	    }
 
 	    if ( goodlink ) {
@@ -930,6 +997,20 @@ delspan(MMIOT *f, int size)
 }
 
 
+/*
+ *  LaTeXspan() -- write out a chunk of text as a section of (unmangled)
+ *                 input for a LaTeX preprocessor
+ */
+static void
+LaTeXspan(MMIOT *f, int size)
+{
+    Qchar('$', f);
+    if ( size > 0 )
+	code(f, cursor(f)-1, size);
+    Qchar('$', f);
+}
+
+
 /*  codespan() -- write out a chunk of text as code, trimming one
  *                space off the front and/or back as appropriate.
  */
@@ -977,19 +1058,20 @@ static int
 maybe_address(char *p, int size)
 {
     int ok = 0;
+    char *q = p;
 
-    for ( ;size && (isalnum(*p) || strchr("._-+*", *p)); ++p, --size)
+    for ( ;size && (isalnum(*q) || strchr("._-+*", *q)); ++q, --size)
 	;
 
-    if ( ! (size && *p == '@') )
+    if ( ! (size && q > p && *q == '@') )
 	return 0;
 
-    --size, ++p;
+    --size, ++q;
 
-    if ( size && *p == '.' ) return 0;
+    if ( size && *q == '.' ) return 0;
 
-    for ( ;size && (isalnum(*p) || strchr("._-+", *p)); ++p, --size )
-	if ( *p == '.' && size > 1 ) ok = 1;
+    for ( ;size && (isalnum(*q) || strchr("._-+", *q)); ++q, --size )
+	if ( *q == '.' && size > 1 ) ok = 1;
 
     return size ? 0 : ok;
 }
@@ -1093,7 +1175,7 @@ maybe_tag_or_link(MMIOT *f)
 	}
 	else {
 	    int i;
-	    
+
 	    if ( forbidden_tag(f) )
 		return 0;
 
@@ -1320,10 +1402,12 @@ tickhandler(MMIOT *f, int tickchar, int minticks, int allow_space, spanhandler s
 	    tick = endticks;
 	}
 
-	shift(f, tick);
-	(*spanner)(f,size);
-	shift(f, size+tick-1);
-	return 1;
+	if ( size > 0 ) {
+	    shift(f, tick);
+	    (*spanner)(f,size);
+	    shift(f, size+tick-1);
+	    return 1;
+	}
     }
     return 0;
 }
@@ -1400,7 +1484,7 @@ text(MMIOT *f)
 			    int here = mmiottell(f);
 			    pull(f);
 
-			    if ( (len = parenthetical('(',')',f)) <= 0 ) {
+			    if ( (len = parenthetical('(',')',f)) == EOF ) {
 				mmiotseek(f,here);
 				Qchar(c, f);
 				break;
@@ -1532,15 +1616,18 @@ text(MMIOT *f)
 			Qchar(c, f);
 		    break;
 
-	case '$':   if ( is_flag_set(&f->flags, MKD_LATEX)
-				    && !is_flag_set(&f->flags, MKD_STRICT)
-				    && (peek(f, 1) == '$') ) {
-			pull(f);
-			if ( mathhandler(f, '$', '$') )
+	case '$':   if ( is_flag_set(&f->flags, MKD_LATEX) && !is_flag_set(&f->flags, MKD_STRICT) ) {
+			if ( peek(f,1) == '$' ) {
+			    pull(f);
+			    if ( mathhandler(f, '$', '$') ) {
+				break;
+			    }
+			    shift(f,-1); /* push back the second '$' */ 
+			}
+			else if ( tickhandler(f,c,1,1,LaTeXspan) )
 			    break;
-			Qchar('$', f);
 		    }
-		    /* fall through to default */
+		    /* fall through to default: */
 
 	default:    f->last = c;
 		    Qchar(c, f);
@@ -1596,7 +1683,7 @@ splat(Line *p, char *block, Istring align, int force, MMIOT *f)
 
 
     ___mkd_tidy(&p->text);
-    if ( T(p->text)[S(p->text)-1] == '|' )
+    if ( S(p->text) > 0 && (T(p->text)[S(p->text)-1] == '|') )
 	--S(p->text);
 
     Qstring("<tr>\n", f);
@@ -1702,7 +1789,7 @@ printtable(Paragraph *pp, MMIOT *f)
 static int
 code_callback(Line *t, char *lang, int fenced, Line **ret, MMIOT *f)
 {
-    if ( f->cb->e_codefmt ) {
+    if ( f && f->cb->e_codefmt.func ) {
 	/* external code block formatter;  copy the text into a buffer,
 	 * call the formatter to style it, then dump that styled text
 	 * directly to the queue
@@ -1725,14 +1812,13 @@ code_callback(Line *t, char *lang, int fenced, Line **ret, MMIOT *f)
 	text[copy_p] = 0;
 
 
-	fmt = (*(f->cb->e_codefmt))(text, copy_p, (lang && lang[0]) ? lang : 0);
+	fmt = (*(f->cb->e_codefmt.func))(text, copy_p, (lang && lang[0]) ? lang : 0);
 	free(text);
 
 	if ( fmt ) {
 	    Qwrite(fmt, strlen(fmt), f);
-	    if ( f->cb->e_free )
-		(*(f->cb->e_free))(fmt, f->cb->e_data);
 	    *ret = t;
+	    if ( f->cb->e_codefmt.free ) (*f->cb->e_codefmt.free)(fmt, strlen(fmt), f);
 	    return 1;
 	}
     }
