@@ -1,6 +1,6 @@
 /* markdown: a C implementation of John Gruber's Markdown markup language.
  *
- * Copyright (C) 2007 David L Parsons.
+ * Copyright (C) 2007 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -105,7 +105,7 @@ ___mkd_tidy(Cstring *t)
 static struct kw comment = { "!--", 3, 0 };
 
 static struct kw *
-isopentag(Line *p)
+isopentag(MMIOT *doc, Line *p)
 {
     int i=0, len;
     char *line;
@@ -133,7 +133,7 @@ isopentag(Line *p)
 	;
 
 
-    return mkd_search_tags(T(p->text)+1, i-1);
+    return mkd_search_tags(doc, T(p->text)+1, i-1);
 }
 
 
@@ -491,12 +491,12 @@ is_extra_dt(Line *t, int *clip, mkd_flag_t* flags)
     return 0;
 }
 
-	
+
 static Line *
 isdefinition(Line *t, int *clip, int *list_type, mkd_flag_t *flags)
 {
     Line *ret;
-    
+
     if ( !is_flag_set(flags, MKD_STRICT) ) {
 	if ( is_flag_set(flags, MKD_DLDISCOUNT) && (ret = is_discount_dt(t,clip,flags)) ) {
 	    *list_type = 1;
@@ -522,7 +522,7 @@ islist(Line *t, int *clip, mkd_flag_t *flags, int *list_type)
 
     if ( isdefinition(t,clip,list_type,flags) )
 	return DL;
-	
+
     if ( strchr("*-+", T(t->text)[t->dle]) && isspace(T(t->text)[t->dle+1]) ) {
 	i = nextnonblank(t, t->dle+1);
 	*clip = (i > 4) ? 4 : i;
@@ -578,16 +578,21 @@ headerblock(Paragraph *pp, int htyp)
 	     * the leading and trailing `#`'s
 	     */
 
-	    for (i=0; (T(p->text)[i] == T(p->text)[0]) && (i < S(p->text)-1)
-						       && (i < 6); i++)
+	    for (i=0; (T(p->text)[i] == T(p->text)[0]) && (i < S(p->text)-1); i++)
 		;
 
-	    pp->hnumber = i;
+	    pp->hnumber = (i > 6) ? 6 : i;;
 
 	    while ( (i < S(p->text)) && isspace(T(p->text)[i]) )
 		++i;
 
 	    CLIP(p->text, 0, i);
+
+	    for (i=S(p->text); (i > 0) && isspace(T(p->text)[i-1]); --i)
+		;
+	    S(p->text) = i;
+	    T(p->text)[i] = 0;
+
 	    UNCHECK(p);
 
 	    for (j=S(p->text); (j > 1) && (T(p->text)[j-1] == '#'); --j)
@@ -596,7 +601,10 @@ headerblock(Paragraph *pp, int htyp)
 	    while ( j && isspace(T(p->text)[j-1]) )
 		--j;
 
-	    S(p->text) = j;
+	    if ( j < S(p->text) ) {
+		T(p->text)[j] = 0;
+		S(p->text) = j;
+	    }
 
 	    ret = p->next;
 	    p->next = 0;
@@ -764,7 +772,8 @@ szmarkerclass(char *p)
  * check if the first line of a quoted block is the special div-not-quote
  * marker %[kind:]name%
  */
-#define iscsschar(c) (isalpha(c) || (c == '-') || (c == '_') )
+
+#define iscsschar(c) (isalpha(c) || (c == '-') || (c == '_') || (c == ' ') )
 
 static int
 isdivmarker(Line *p, int start, mkd_flag_t *flags)
@@ -1067,6 +1076,35 @@ extrablock(Line *p)
 }
 
 
+#if 0
+/*
+ * get the width & height of a footnote image
+ */
+static void
+footnote_height_and_width(char *s, struct footnote *foot)
+{
+    int i = 0;
+
+    if ( isdigit(s[i]) ) { 
+	while ( isdigit(s[i]) )
+	    EXPAND(foot->width) = s[i++];
+	/* specialcase for % width */
+	if ( s[i] == '%' )
+	    EXPAND(foot->width) = '%';
+	EXPAND(foot->width) = 0;
+    }
+    if ( s[i] == 'x' ) {
+	i++;
+	while (isdigit(s[i]) )
+	    EXPAND(foot->height) = s[i++];
+	if ( s[i] == '%' )
+	    EXPAND(foot->height) = '%';
+	EXPAND(foot->height) = 0;
+    }
+}
+#endif
+
+
 /*
  * add a new (image or link) footnote to the footnote table
  */
@@ -1082,8 +1120,11 @@ addfootnote(Line *p, MMIOT* f)
     CREATE(foot->tag);
     CREATE(foot->link);
     CREATE(foot->title);
+    foot->fn_flags = 0;
     foot->text = 0;
-    foot->fn_flags = foot->height = foot->width = 0;
+    CREATE(foot->height);
+    CREATE(foot->width);
+    CREATE(foot->extended_attr);
 
     /* keep the footnote label */
     for (j=i=p->dle+1; T(p->text)[j] != ']'; j++)
@@ -1117,11 +1158,23 @@ addfootnote(Line *p, MMIOT* f)
     S(foot->link)--;
     j = nextnonblank(p,j);
 
+#if 0
     if ( T(p->text)[j] == '=' ) {
-	sscanf(T(p->text)+j, "=%dx%d", &foot->width, &foot->height);
+	footnote_height_and_width(T(p->text)+j, foot);
 	j = nextblank(p, j);
 	j = nextnonblank(p,j);
     }
+#endif
+
+    if ( is_flag_set(&f->flags, MKD_EXTENDED_ATTR) && (T(p->text)[j] == '{') ) {
+	for ( i=j; T(p->text)[i] && ( T(p->text)[i] != '}' ); i++ )
+	    ;
+	if ( T(p->text)[i] == '}' ) {
+	    for ( j++; j < i; j++ )
+		EXPAND(foot->extended_attr) = T(p->text)[j];
+	    j++;
+	}
+    }
 
 
     if ( (j >= S(p->text)) && np && np->dle && tgood(T(np->text)[np->dle]) ) {
@@ -1218,7 +1271,7 @@ compile_document(Line *ptr, MMIOT *f)
     int previous_was_break = 1;
 
     while ( ptr ) {
-	if ( !is_flag_set(&(f->flags), MKD_NOHTML) && (tag = isopentag(ptr)) ) {
+	if ( !is_flag_set(&(f->flags), MKD_NOHTML) && (tag = isopentag(f, ptr)) ) {
 	    int blocktype;
 	    /* If we encounter a html/style block, compile and save all
 	     * of the cached source BEFORE processing the html/style.
@@ -1417,7 +1470,7 @@ compile(Line *ptr, int toplevel, MMIOT *f)
 					 * processing with textblock()
 					 */
 
-	    if ( !is_flag_set(&(f->flags), MKD_NOHTML) && (tag = isopentag(ptr)) ) {
+	    if ( !is_flag_set(&(f->flags), MKD_NOHTML) && (tag = isopentag(f, ptr)) ) {
 		/* possibly an html block
 		 */
 
@@ -1476,18 +1529,13 @@ mkd_compile(Document *doc, mkd_flag_t* flags)
     }
 
     doc->compiled = 1;
-    memset(doc->ctx, 0, sizeof(MMIOT) );
+    
+    ___mkd_initmmiot(doc->ctx, NULL, flags);
+    
     doc->ctx->ref_prefix= doc->ref_prefix;
     doc->ctx->cb        = &(doc->cb);
-    if (flags)
-	COPY_FLAGS(doc->ctx->flags, *flags);
-    else
-	mkd_init_flags(&doc->ctx->flags);
-    CREATE(doc->ctx->in);
-    doc->ctx->footnotes = malloc(sizeof doc->ctx->footnotes[0]);
-    doc->ctx->footnotes->reference = 0;
-    CREATE(doc->ctx->footnotes->note);
 
+    CREATE(doc->ctx->in);
 
     mkd_initialize();
 
