@@ -15,7 +15,7 @@
  *     ( read syntax OR syntax.text, write syntax.html )
  */
 /*
- * Copyright (C) 2007 David L Parsons.
+ * Copyright (C) 2007 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -74,7 +74,7 @@ enum { GFM, ADD_CSS, ADD_HEADER, ADD_FOOTER };
 struct h_opt opts[] = {
     { GFM,           "gfm",'G', 0,       "Github style markdown" },
     { ADD_CSS,       "css", 0, "url",    "Additional css for this page" },
-    { ADD_HEADER, "header", 0, "header", "Additonal headers for this page" },
+    { ADD_HEADER, "header", 0, "header", "Additional headers for this page" },
     { ADD_FOOTER, "footer", 0, "footer", "Additional footers for this page" },
 };
 #define NROPTS (sizeof opts/sizeof opts[0])
@@ -85,8 +85,7 @@ extern char* mkd_h1_title(MMIOT *);
 
 
 int
-main(argc, argv)
-char **argv;
+main(int argc, char **argv)
 {
     char *h;
     char *source = 0, *dest = 0;
@@ -195,7 +194,7 @@ char **argv;
     /* print a header */
 
     fprintf(output,
-	"<!doctype html public \"-//W3C//DTD HTML 4.0 Transitional //EN\">\n"
+	"<!doctype html>\n"
 	"<html>\n"
 	"<head>\n"
 	"  <meta name=\"GENERATOR\" content=\"mkd2html %s\">\n", markdown_version);
