@@ -1,6 +1,6 @@
 /* markdown: a C implementation of John Gruber's Markdown markup language.
  *
- * Copyright (C) 2007-2011 David L Parsons.
+ * Copyright (C) 2007-2011 Jessica L Parsons.
  * The redistribution terms are provided in the COPYRIGHT file that must
  * be distributed with this source code.
  */
@@ -44,7 +44,7 @@ static struct _opt {
     { "image",         "images",                     0, 1, 0, 1, MKD_NOIMAGE  },
     { "links",         "links",                      0, 1, 0, 1, MKD_NOLINKS  },
     { "strict",        "conform to the markdown standard",  0, 0, 0, 1, MKD_STRICT   },
-    { "relax",         "conform to the markdown standard",  0, 1, 1, 1, MKD_STRICT   },
+    { "relax",         "(don't) conform to the markdown standard",  0, 1, 1, 1, MKD_STRICT   },
     { "standard",      "conform to the markdown standard", 0, 0, 1, 1, MKD_STRICT },
     { "tables",        "tables",                     0, 1, 0, 1, MKD_NOTABLES },
     { "header",        "pandoc-style headers",       0, 1, 0, 1, MKD_NOHEADER },
@@ -78,6 +78,10 @@ static struct _opt {
     { "regular-listitem","github-style check items", 0, 0, 1, 1, MKD_NORMAL_LISTITEM } ,
     { "definitionlist","both discount & markdown extra definition lists", 1 },
     { "dlist",         "both discount & markdown extra definition lists", 1, 0, 1 },
+    { "alt_as_title",  "use the alt text as a title if there isn't one (images)", 0, 0, 0, 1, MKD_ALT_AS_TITLE },
+    { "extended_attr", "allow extended attributes on links", 0, 0, 1, 1, MKD_EXTENDED_ATTR },
+    { "extended_attributes", "allow extended attributes on links", 0, 0, 0, 1, MKD_EXTENDED_ATTR },
+    { "html5",         "handle html5 tags (obsolete?)", 0, 0, 0, 1, MKD_HTML5 },
 } ;
 
 #define NR(x)	(sizeof x / sizeof x[0])
@@ -104,20 +108,26 @@ show_flags(int byname, int verbose, mkd_flag_t *flags)
     int i;
 
     if ( byname ) {
+	int size, len=0;
+
 	qsort(opts, NR(opts), sizeof(opts[0]), (stfu)sort_by_name);
-    
+
+	for (i=0; i < NR(opts); i++)
+	    if ( (size=strlen(opts[i].name)) > len )
+		len = size;
+
 	for (i=0; i < NR(opts); i++) {
 	    if ( opts[i].alias && !verbose )
 		continue;
 	    if ( (flags==0) || is_flag_set(flags, opts[i].flag) )
-		fprintf(stderr, "%16s : %s\n", opts[i].name, opts[i].desc);
+		fprintf(stderr, "%*s : %s\n", len+1, opts[i].name, opts[i].desc);
 	}
     }
     else {
 	qsort(opts, NR(opts), sizeof(opts[0]), (stfu)sort_by_flag);
-	
+
 	for (i=0; i < NR(opts) && i < 8*sizeof(DWORD); i++) {
-	    
+
 	    if ( opts[i].special || opts[i].alias )
 		continue;
 
@@ -144,6 +154,18 @@ handle_special(mkd_flag_t *flags, char *opt, int enable)
 	}
 }
 
+static void
+set_dlist(mkd_flag_t *flags, int enable)
+{
+    if ( enable ) {
+	set_mkd_flag(flags, MKD_DLDISCOUNT);
+	set_mkd_flag(flags, MKD_DLEXTRA);
+    }
+    else {
+	clear_mkd_flag(flags, MKD_DLDISCOUNT);
+	clear_mkd_flag(flags, MKD_DLEXTRA);
+    }
+}
 
 char *
 mkd_set_flag_string(mkd_flag_t *flags, char *optionstring)
@@ -194,16 +216,3 @@ mkd_set_flag_string(mkd_flag_t *flags, char *optionstring)
     }
     return 0;
 }
-
-static void
-set_dlist(mkd_flag_t *flags, int enable)
-{
-    if ( enable ) {
-	set_mkd_flag(flags, MKD_DLDISCOUNT);
-	set_mkd_flag(flags, MKD_DLEXTRA);
-    }
-    else {
-	clear_mkd_flag(flags, MKD_DLDISCOUNT);
-	clear_mkd_flag(flags, MKD_DLEXTRA);
-    }
-}
